@@ -17,6 +17,14 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+/*
+ * IF YOU MODIFY THIS FILE PLEASE RUN `make check` and ensure that
+ * the test_zmq.c unit test is still working.  There are dependencies
+ * between the two that are extremely fragile.  My understanding
+ * is that there is specialized ownership of the cb pointer based
+ * upon what is happening.  Those assumptions are supposed to be
+ * tested in the test_zmq.c
+ */
 #include <zebra.h>
 #include <zmq.h>
 
@@ -26,7 +34,7 @@
 #include "log.h"
 #include "lib_errors.h"
 
-DEFINE_MTYPE_STATIC(LIB, ZEROMQ_CB, "ZeroMQ callback")
+DEFINE_MTYPE_STATIC(LIB, ZEROMQ_CB, "ZeroMQ callback");
 
 /* libzmq's context */
 void *frrzmq_context = NULL;
@@ -76,7 +84,10 @@ static int frrzmq_read_msg(struct thread *t)
 			break;
 
 		if (cb->read.cb_msg) {
+			cb->in_cb = true;
 			cb->read.cb_msg(cb->read.arg, cb->zmqsock);
+			cb->in_cb = false;
+
 			read = 1;
 
 			if (cb->read.cancelled) {
@@ -84,7 +95,8 @@ static int frrzmq_read_msg(struct thread *t)
 						    ZMQ_POLLOUT);
 				cb->read.thread = NULL;
 				if (cb->write.cancelled && !cb->write.thread)
-					XFREE(MTYPE_ZEROMQ_CB, cb);
+					XFREE(MTYPE_ZEROMQ_CB, *cbp);
+
 				return 0;
 			}
 			continue;
@@ -104,15 +116,19 @@ static int frrzmq_read_msg(struct thread *t)
 			}
 			read = 1;
 
+			cb->in_cb = true;
 			cb->read.cb_part(cb->read.arg, cb->zmqsock, &msg,
 					 partno);
+			cb->in_cb = false;
+
 			if (cb->read.cancelled) {
 				zmq_msg_close(&msg);
 				frrzmq_check_events(cbp, &cb->write,
 						    ZMQ_POLLOUT);
 				cb->read.thread = NULL;
 				if (cb->write.cancelled && !cb->write.thread)
-					XFREE(MTYPE_ZEROMQ_CB, cb);
+					XFREE(MTYPE_ZEROMQ_CB, *cbp);
+
 				return 0;
 			}
 
@@ -135,27 +151,26 @@ static int frrzmq_read_msg(struct thread *t)
 	if (read)
 		frrzmq_check_events(cbp, &cb->write, ZMQ_POLLOUT);
 
-	funcname_thread_add_read_write(
-		THREAD_READ, t->master, frrzmq_read_msg, cbp, cb->fd,
-		&cb->read.thread, t->funcname, t->schedfrom, t->schedfrom_line);
+	thread_add_read(t->master, frrzmq_read_msg, cbp,
+			cb->fd, &cb->read.thread);
 	return 0;
 
 out_err:
-	flog_err(LIB_ERR_ZMQ, "ZeroMQ read error: %s(%d)", strerror(errno),
-		  errno);
+	flog_err(EC_LIB_ZMQ, "ZeroMQ read error: %s(%d)", strerror(errno),
+		 errno);
 	if (cb->read.cb_error)
 		cb->read.cb_error(cb->read.arg, cb->zmqsock);
 	return 1;
 }
 
-int funcname_frrzmq_thread_add_read(struct thread_master *master,
-				    void (*msgfunc)(void *arg, void *zmqsock),
-				    void (*partfunc)(void *arg, void *zmqsock,
-						     zmq_msg_t *msg,
-						     unsigned partnum),
-				    void (*errfunc)(void *arg, void *zmqsock),
-				    void *arg, void *zmqsock,
-				    struct frrzmq_cb **cbp, debugargdef)
+int _frrzmq_thread_add_read(const struct xref_threadsched *xref,
+			    struct thread_master *master,
+			    void (*msgfunc)(void *arg, void *zmqsock),
+			    void (*partfunc)(void *arg, void *zmqsock,
+					     zmq_msg_t *msg, unsigned partnum),
+			    void (*errfunc)(void *arg, void *zmqsock),
+			    void *arg, void *zmqsock,
+			    struct frrzmq_cb **cbp)
 {
 	int fd, events;
 	size_t len;
@@ -176,10 +191,7 @@ int funcname_frrzmq_thread_add_read(struct thread_master *master,
 		cb = *cbp;
 	else {
 		cb = XCALLOC(MTYPE_ZEROMQ_CB, sizeof(struct frrzmq_cb));
-		if (!cb)
-			return -1;
-
-		cb->write.cancelled = 1;
+		cb->write.cancelled = true;
 		*cbp = cb;
 	}
 
@@ -189,20 +201,17 @@ int funcname_frrzmq_thread_add_read(struct thread_master *master,
 	cb->read.cb_msg = msgfunc;
 	cb->read.cb_part = partfunc;
 	cb->read.cb_error = errfunc;
-	cb->read.cancelled = 0;
+	cb->read.cancelled = false;
+	cb->in_cb = false;
 
 	if (events & ZMQ_POLLIN) {
-		if (cb->read.thread) {
-			thread_cancel(cb->read.thread);
-			cb->read.thread = NULL;
-		}
-		funcname_thread_add_event(master, frrzmq_read_msg, cbp, fd,
-					  &cb->read.thread, funcname, schedfrom,
-					  fromln);
+		thread_cancel(&cb->read.thread);
+
+		thread_add_event(master, frrzmq_read_msg, cbp, fd,
+				  &cb->read.thread);
 	} else
-		funcname_thread_add_read_write(
-			THREAD_READ, master, frrzmq_read_msg, cbp, fd,
-			&cb->read.thread, funcname, schedfrom, fromln);
+		thread_add_read(master, frrzmq_read_msg, cbp, fd,
+				&cb->read.thread);
 	return 0;
 }
 
@@ -231,14 +240,18 @@ static int frrzmq_write_msg(struct thread *t)
 			break;
 
 		if (cb->write.cb_msg) {
+			cb->in_cb = true;
 			cb->write.cb_msg(cb->write.arg, cb->zmqsock);
+			cb->in_cb = false;
+
 			written = 1;
 
 			if (cb->write.cancelled) {
 				frrzmq_check_events(cbp, &cb->read, ZMQ_POLLIN);
 				cb->write.thread = NULL;
 				if (cb->read.cancelled && !cb->read.thread)
-					XFREE(MTYPE_ZEROMQ_CB, cb);
+					XFREE(MTYPE_ZEROMQ_CB, *cbp);
+
 				return 0;
 			}
 			continue;
@@ -248,24 +261,23 @@ static int frrzmq_write_msg(struct thread *t)
 	if (written)
 		frrzmq_check_events(cbp, &cb->read, ZMQ_POLLIN);
 
-	funcname_thread_add_read_write(THREAD_WRITE, t->master,
-				       frrzmq_write_msg, cbp, cb->fd,
-				       &cb->write.thread, t->funcname,
-				       t->schedfrom, t->schedfrom_line);
+	thread_add_write(t->master, frrzmq_write_msg, cbp,
+			 cb->fd, &cb->write.thread);
 	return 0;
 
 out_err:
-	flog_err(LIB_ERR_ZMQ, "ZeroMQ write error: %s(%d)", strerror(errno),
-		  errno);
+	flog_err(EC_LIB_ZMQ, "ZeroMQ write error: %s(%d)", strerror(errno),
+		 errno);
 	if (cb->write.cb_error)
 		cb->write.cb_error(cb->write.arg, cb->zmqsock);
 	return 1;
 }
-int funcname_frrzmq_thread_add_write(struct thread_master *master,
-				     void (*msgfunc)(void *arg, void *zmqsock),
-				     void (*errfunc)(void *arg, void *zmqsock),
-				     void *arg, void *zmqsock,
-				     struct frrzmq_cb **cbp, debugargdef)
+
+int _frrzmq_thread_add_write(const struct xref_threadsched *xref,
+			     struct thread_master *master,
+			     void (*msgfunc)(void *arg, void *zmqsock),
+			     void (*errfunc)(void *arg, void *zmqsock),
+			     void *arg, void *zmqsock, struct frrzmq_cb **cbp)
 {
 	int fd, events;
 	size_t len;
@@ -286,10 +298,7 @@ int funcname_frrzmq_thread_add_write(struct thread_master *master,
 		cb = *cbp;
 	else {
 		cb = XCALLOC(MTYPE_ZEROMQ_CB, sizeof(struct frrzmq_cb));
-		if (!cb)
-			return -1;
-
-		cb->read.cancelled = 1;
+		cb->read.cancelled = true;
 		*cbp = cb;
 	}
 
@@ -299,20 +308,17 @@ int funcname_frrzmq_thread_add_write(struct thread_master *master,
 	cb->write.cb_msg = msgfunc;
 	cb->write.cb_part = NULL;
 	cb->write.cb_error = errfunc;
-	cb->write.cancelled = 0;
+	cb->write.cancelled = false;
+	cb->in_cb = false;
 
 	if (events & ZMQ_POLLOUT) {
-		if (cb->write.thread) {
-			thread_cancel(cb->write.thread);
-			cb->write.thread = NULL;
-		}
-		funcname_thread_add_event(master, frrzmq_write_msg, cbp, fd,
-					  &cb->write.thread, funcname,
-					  schedfrom, fromln);
+		thread_cancel(&cb->write.thread);
+
+		_thread_add_event(xref, master, frrzmq_write_msg, cbp, fd,
+				  &cb->write.thread);
 	} else
-		funcname_thread_add_read_write(
-			THREAD_WRITE, master, frrzmq_write_msg, cbp, fd,
-			&cb->write.thread, funcname, schedfrom, fromln);
+		thread_add_write(master, frrzmq_write_msg, cbp, fd,
+				 &cb->write.thread);
 	return 0;
 }
 
@@ -320,13 +326,18 @@ void frrzmq_thread_cancel(struct frrzmq_cb **cb, struct cb_core *core)
 {
 	if (!cb || !*cb)
 		return;
-	core->cancelled = 1;
-	if (core->thread) {
-		thread_cancel(core->thread);
-		core->thread = NULL;
-	}
+	core->cancelled = true;
+	thread_cancel(&core->thread);
+
+	/* If cancelled from within a callback, don't try to free memory
+	 * in this path.
+	 */
+	if ((*cb)->in_cb)
+		return;
+
+	/* Ok to free the callback context if no more ... context. */
 	if ((*cb)->read.cancelled && !(*cb)->read.thread
-	    && (*cb)->write.cancelled && (*cb)->write.thread)
+	    && (*cb)->write.cancelled && ((*cb)->write.thread == NULL))
 		XFREE(MTYPE_ZEROMQ_CB, *cb);
 }
 
@@ -346,12 +357,16 @@ void frrzmq_check_events(struct frrzmq_cb **cbp, struct cb_core *core,
 	len = sizeof(events);
 	if (zmq_getsockopt(cb->zmqsock, ZMQ_EVENTS, &events, &len))
 		return;
-	if (events & event && core->thread && !core->cancelled) {
+	if ((events & event) && core->thread && !core->cancelled) {
 		struct thread_master *tm = core->thread->master;
-		thread_cancel(core->thread);
-		core->thread = NULL;
-		thread_add_event(tm, (event == ZMQ_POLLIN ? frrzmq_read_msg
-							  : frrzmq_write_msg),
-				 cbp, cb->fd, &core->thread);
+
+		thread_cancel(&core->thread);
+
+		if (event == ZMQ_POLLIN)
+			thread_add_event(tm, frrzmq_read_msg,
+					 cbp, cb->fd, &core->thread);
+		else
+			thread_add_event(tm, frrzmq_write_msg,
+					 cbp, cb->fd, &core->thread);
 	}
 }
