@@ -54,6 +54,7 @@
 #include "ospfd/ospf_route.h"
 #include "ospfd/ospf_ase.h"
 #include "ospfd/ospf_zebra.h"
+#include "ospfd/ospf_errors.h"
 
 #include "ospfd/ospf_api.h"
 #include "ospfd/ospf_apiserver.h"
@@ -152,7 +153,8 @@ int ospf_apiserver_init(void)
 		NULL,		  /* ospf_apiserver_lsa_refresher */
 		ospf_apiserver_lsa_update, ospf_apiserver_lsa_delete);
 	if (rc != 0) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_OPAQUE_REGISTRATION,
 			"ospf_apiserver_init: Failed to register opaque type [0/0]");
 	}
 
@@ -180,7 +182,7 @@ void ospf_apiserver_term(void)
 
 	/* Free client list itself */
 	if (apiserver_list)
-		list_delete_and_null(&apiserver_list);
+		list_delete(&apiserver_list);
 
 	/* Free wildcard list */
 	/* XXX  */
@@ -315,21 +317,12 @@ void ospf_apiserver_free(struct ospf_apiserver *apiserv)
 	struct listnode *node;
 
 	/* Cancel read and write threads. */
-	if (apiserv->t_sync_read) {
-		thread_cancel(apiserv->t_sync_read);
-	}
+	thread_cancel(&apiserv->t_sync_read);
 #ifdef USE_ASYNC_READ
-	if (apiserv->t_async_read) {
-		thread_cancel(apiserv->t_async_read);
-	}
+	thread_cancel(&apiserv->t_async_read);
 #endif /* USE_ASYNC_READ */
-	if (apiserv->t_sync_write) {
-		thread_cancel(apiserv->t_sync_write);
-	}
-
-	if (apiserv->t_async_write) {
-		thread_cancel(apiserv->t_async_write);
-	}
+	thread_cancel(&apiserv->t_sync_write);
+	thread_cancel(&apiserv->t_async_write);
 
 	/* Unregister all opaque types that application registered
 	   and flush opaque LSAs if still in LSDB. */
@@ -385,8 +378,8 @@ int ospf_apiserver_read(struct thread *thread)
 		apiserv->t_sync_read = NULL;
 
 		if (IS_DEBUG_OSPF_EVENT)
-			zlog_debug("API: ospf_apiserver_read: Peer: %s/%u",
-				   inet_ntoa(apiserv->peer_sync.sin_addr),
+			zlog_debug("API: ospf_apiserver_read: Peer: %pI4/%u",
+				   &apiserv->peer_sync.sin_addr,
 				   ntohs(apiserv->peer_sync.sin_port));
 	}
 #ifdef USE_ASYNC_READ
@@ -395,8 +388,8 @@ int ospf_apiserver_read(struct thread *thread)
 		apiserv->t_async_read = NULL;
 
 		if (IS_DEBUG_OSPF_EVENT)
-			zlog_debug("API: ospf_apiserver_read: Peer: %s/%u",
-				   inet_ntoa(apiserv->peer_async.sin_addr),
+			zlog_debug("API: ospf_apiserver_read: Peer: %pI4/%u",
+				   &apiserv->peer_async.sin_addr,
 				   ntohs(apiserv->peer_async.sin_port));
 	}
 #endif /* USE_ASYNC_READ */
@@ -453,8 +446,8 @@ int ospf_apiserver_sync_write(struct thread *thread)
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_debug("API: ospf_apiserver_sync_write: Peer: %s/%u",
-			   inet_ntoa(apiserv->peer_sync.sin_addr),
+		zlog_debug("API: ospf_apiserver_sync_write: Peer: %pI4/%u",
+			   &apiserv->peer_sync.sin_addr,
 			   ntohs(apiserv->peer_sync.sin_port));
 
 	/* Check whether there is really a message in the fifo. */
@@ -517,8 +510,8 @@ int ospf_apiserver_async_write(struct thread *thread)
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_debug("API: ospf_apiserver_async_write: Peer: %s/%u",
-			   inet_ntoa(apiserv->peer_async.sin_addr),
+		zlog_debug("API: ospf_apiserver_async_write: Peer: %pI4/%u",
+			   &apiserv->peer_async.sin_addr,
 			   ntohs(apiserv->peer_async.sin_port));
 
 	/* Check whether there is really a message in the fifo. */
@@ -643,8 +636,8 @@ int ospf_apiserver_accept(struct thread *thread)
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_debug("API: ospf_apiserver_accept: New peer: %s/%u",
-			   inet_ntoa(peer_sync.sin_addr),
+		zlog_debug("API: ospf_apiserver_accept: New peer: %pI4/%u",
+			   &peer_sync.sin_addr,
 			   ntohs(peer_sync.sin_port));
 
 	/* Create new socket for asynchronous messages. */
@@ -655,8 +648,8 @@ int ospf_apiserver_accept(struct thread *thread)
 	 */
 	if (ntohs(peer_async.sin_port) == ospf_apiserver_getport()) {
 		zlog_warn(
-			"API: ospf_apiserver_accept: Peer(%s/%u): Invalid async port number?",
-			inet_ntoa(peer_async.sin_addr),
+			"API: ospf_apiserver_accept: Peer(%pI4/%u): Invalid async port number?",
+			&peer_async.sin_addr,
 			ntohs(peer_async.sin_port));
 		close(new_sync_sock);
 		return -1;
@@ -867,7 +860,8 @@ int ospf_apiserver_register_opaque_type(struct ospf_apiserver *apiserv,
 		NULL /* ospf_apiserver_lsa_delete */);
 
 	if (rc != 0) {
-		zlog_warn("Failed to register opaque type [%d/%d]", lsa_type,
+		flog_warn(EC_OSPF_OPAQUE_REGISTRATION,
+			  "Failed to register opaque type [%d/%d]", lsa_type,
 			  opaque_type);
 		return OSPF_API_OPAQUETYPEINUSE;
 	}
@@ -886,8 +880,7 @@ int ospf_apiserver_register_opaque_type(struct ospf_apiserver *apiserv,
 
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug(
-			"API: Add LSA-type(%d)/Opaque-type(%d) into"
-			" apiserv(%p), total#(%d)",
+			"API: Add LSA-type(%d)/Opaque-type(%d) into apiserv(%p), total#(%d)",
 			lsa_type, opaque_type, (void *)apiserv,
 			listcount(apiserv->opaque_types));
 
@@ -917,8 +910,7 @@ int ospf_apiserver_unregister_opaque_type(struct ospf_apiserver *apiserv,
 
 			if (IS_DEBUG_OSPF_EVENT)
 				zlog_debug(
-					"API: Del LSA-type(%d)/Opaque-type(%d)"
-					" from apiserv(%p), total#(%d)",
+					"API: Del LSA-type(%d)/Opaque-type(%d) from apiserv(%p), total#(%d)",
 					lsa_type, opaque_type, (void *)apiserv,
 					listcount(apiserv->opaque_types));
 
@@ -1164,6 +1156,7 @@ int ospf_apiserver_handle_register_event(struct ospf_apiserver *apiserv,
 	struct msg_register_event *rmsg;
 	int rc;
 	uint32_t seqnum;
+	size_t size;
 
 	rmsg = (struct msg_register_event *)STREAM_DATA(msg->s);
 
@@ -1173,16 +1166,17 @@ int ospf_apiserver_handle_register_event(struct ospf_apiserver *apiserv,
 	/* Free existing filter in apiserv. */
 	XFREE(MTYPE_OSPF_APISERVER_MSGFILTER, apiserv->filter);
 	/* Alloc new space for filter. */
+	size = ntohs(msg->hdr.msglen);
+	if (size < OSPF_MAX_LSA_SIZE) {
+
+		apiserv->filter = XMALLOC(MTYPE_OSPF_APISERVER_MSGFILTER, size);
 
-	apiserv->filter =
-		XMALLOC(MTYPE_OSPF_APISERVER_MSGFILTER, ntohs(msg->hdr.msglen));
-	if (apiserv->filter) {
 		/* copy it over. */
-		memcpy(apiserv->filter, &rmsg->filter, ntohs(msg->hdr.msglen));
+		memcpy(apiserv->filter, &rmsg->filter, size);
 		rc = OSPF_API_OK;
-	} else {
+	} else
 		rc = OSPF_API_NOMEMORY;
-	}
+
 	/* Send a reply back to client with return code */
 	rc = ospf_apiserver_send_reply(apiserv, seqnum, rc);
 	return rc;
@@ -1410,8 +1404,8 @@ struct ospf_lsa *ospf_apiserver_opaque_lsa_new(struct ospf_area *area,
 	options |= OSPF_OPTION_O; /* Don't forget to set option bit */
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
-		zlog_debug("LSA[Type%d:%s]: Creating an Opaque-LSA instance",
-			   protolsa->type, inet_ntoa(protolsa->id));
+		zlog_debug("LSA[Type%d:%pI4]: Creating an Opaque-LSA instance",
+			   protolsa->type, &protolsa->id);
 	}
 
 	/* Set opaque-LSA header fields. */
@@ -1427,19 +1421,7 @@ struct ospf_lsa *ospf_apiserver_opaque_lsa_new(struct ospf_area *area,
 	newlsa->length = htons(length);
 
 	/* Create OSPF LSA. */
-	if ((new = ospf_lsa_new()) == NULL) {
-		zlog_warn("ospf_apiserver_opaque_lsa_new: ospf_lsa_new() ?");
-		stream_free(s);
-		return NULL;
-	}
-
-	if ((new->data = ospf_lsa_data_new(length)) == NULL) {
-		zlog_warn(
-			"ospf_apiserver_opaque_lsa_new: ospf_lsa_data_new() ?");
-		ospf_lsa_unlock(&new);
-		stream_free(s);
-		return NULL;
-	}
+	new = ospf_lsa_new_and_data(length);
 
 	new->area = area;
 	new->oi = oi;
@@ -1523,8 +1505,8 @@ int ospf_apiserver_handle_originate_request(struct ospf_apiserver *apiserv,
 	case OSPF_OPAQUE_LINK_LSA:
 		oi = ospf_apiserver_if_lookup_by_addr(omsg->ifaddr);
 		if (!oi) {
-			zlog_warn("apiserver_originate: unknown interface %s",
-				  inet_ntoa(omsg->ifaddr));
+			zlog_warn("apiserver_originate: unknown interface %pI4",
+				  &omsg->ifaddr);
 			rc = OSPF_API_NOSUCHINTERFACE;
 			goto out;
 		}
@@ -1534,8 +1516,8 @@ int ospf_apiserver_handle_originate_request(struct ospf_apiserver *apiserv,
 	case OSPF_OPAQUE_AREA_LSA:
 		area = ospf_area_lookup_by_area_id(ospf, omsg->area_id);
 		if (!area) {
-			zlog_warn("apiserver_originate: unknown area %s",
-				  inet_ntoa(omsg->area_id));
+			zlog_warn("apiserver_originate: unknown area %pI4",
+				  &omsg->area_id);
 			rc = OSPF_API_NOSUCHAREA;
 			goto out;
 		}
@@ -1676,7 +1658,8 @@ int ospf_apiserver_originate1(struct ospf_lsa *lsa)
 
 	/* Install this LSA into LSDB. */
 	if (ospf_lsa_install(ospf, lsa->oi, lsa) == NULL) {
-		zlog_warn("ospf_apiserver_originate1: ospf_lsa_install failed");
+		flog_warn(EC_OSPF_LSA_INSTALL_FAILURE,
+			  "ospf_apiserver_originate1: ospf_lsa_install failed");
 		return -1;
 	}
 
@@ -1741,6 +1724,8 @@ struct ospf_lsa *ospf_apiserver_lsa_refresher(struct ospf_lsa *lsa)
 	struct ospf_lsa *new = NULL;
 	struct ospf *ospf;
 
+	assert(lsa);
+
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	assert(ospf);
 
@@ -1751,6 +1736,7 @@ struct ospf_lsa *ospf_apiserver_lsa_refresher(struct ospf_lsa *lsa)
 			dump_lsa_key(lsa));
 		lsa->data->ls_age =
 			htons(OSPF_LSA_MAXAGE); /* Flush it anyway. */
+		goto out;
 	}
 
 	if (IS_LSA_MAXAGE(lsa)) {
@@ -1784,7 +1770,8 @@ struct ospf_lsa *ospf_apiserver_lsa_refresher(struct ospf_lsa *lsa)
 
 	/* Install LSA into LSDB. */
 	if (ospf_lsa_install(ospf, new->oi, new) == NULL) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_LSA_INSTALL_FAILURE,
 			"ospf_apiserver_lsa_refresher: ospf_lsa_install failed");
 		ospf_lsa_unlock(&new);
 		goto out;
@@ -1799,8 +1786,8 @@ struct ospf_lsa *ospf_apiserver_lsa_refresher(struct ospf_lsa *lsa)
 
 	/* Debug logging. */
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
-		zlog_debug("LSA[Type%d:%s]: Refresh Opaque LSA",
-			   new->data->type, inet_ntoa(new->data->id));
+		zlog_debug("LSA[Type%d:%pI4]: Refresh Opaque LSA",
+			   new->data->type, &new->data->id);
 		ospf_lsa_header_dump(new->data);
 	}
 
@@ -1837,8 +1824,8 @@ int ospf_apiserver_handle_delete_request(struct ospf_apiserver *apiserv,
 	case OSPF_OPAQUE_AREA_LSA:
 		area = ospf_area_lookup_by_area_id(ospf, dmsg->area_id);
 		if (!area) {
-			zlog_warn("ospf_apiserver_lsa_delete: unknown area %s",
-				  inet_ntoa(dmsg->area_id));
+			zlog_warn("ospf_apiserver_lsa_delete: unknown area %pI4",
+				  &dmsg->area_id);
 			rc = OSPF_API_NOSUCHAREA;
 			goto out;
 		}
@@ -1880,8 +1867,8 @@ int ospf_apiserver_handle_delete_request(struct ospf_apiserver *apiserv,
 	old = ospf_lsa_lookup(ospf, area, dmsg->lsa_type, id, ospf->router_id);
 	if (!old) {
 		zlog_warn(
-			"ospf_apiserver_lsa_delete: LSA[Type%d:%s] not in LSDB",
-			dmsg->lsa_type, inet_ntoa(id));
+			"ospf_apiserver_lsa_delete: LSA[Type%d:%pI4] not in LSDB",
+			dmsg->lsa_type, &id);
 		rc = OSPF_API_NOSUCHLSA;
 		goto out;
 	}
@@ -2030,7 +2017,7 @@ int ospf_apiserver_del_if(struct interface *ifp)
 
 	/* zlog_warn for debugging */
 	zlog_warn("ospf_apiserver_del_if");
-	zlog_warn("ifp name=%s status=%d index=%d\n", ifp->name, ifp->status,
+	zlog_warn("ifp name=%s status=%d index=%d", ifp->name, ifp->status,
 		  ifp->ifindex);
 
 	oi = ospf_apiserver_if_lookup_by_ifp(ifp);
@@ -2072,7 +2059,8 @@ void ospf_apiserver_nsm_change(struct ospf_neighbor *nbr, int old_status)
 	}
 }
 
-void ospf_apiserver_show_info(struct vty *vty, struct ospf_lsa *lsa)
+void ospf_apiserver_show_info(struct vty *vty, struct json_object *json,
+			      struct ospf_lsa *lsa)
 {
 	struct opaque_lsa {
 		struct lsa_header header;
@@ -2083,6 +2071,9 @@ void ospf_apiserver_show_info(struct vty *vty, struct ospf_lsa *lsa)
 	struct opaque_lsa *olsa;
 	int opaquelen;
 
+	if (json)
+		return;
+
 	olsa = (struct opaque_lsa *)lsa->data;
 
 	if (VALID_OPAQUE_INFO_LEN(lsa->data))
@@ -2116,7 +2107,6 @@ void ospf_apiserver_show_info(struct vty *vty, struct ospf_lsa *lsa)
 		for (i = 0; i < opaquelen; i++) {
 			zlog_debug("0x%x ", olsa->data[i]);
 		}
-		zlog_debug("\n");
 	}
 	return;
 }
@@ -2332,14 +2322,12 @@ void ospf_apiserver_clients_notify_ism_change(struct ospf_interface *oi)
 void ospf_apiserver_clients_notify_nsm_change(struct ospf_neighbor *nbr)
 {
 	struct msg *msg;
-	struct in_addr ifaddr = {.s_addr = 0L};
+	struct in_addr ifaddr;
 	struct in_addr nbraddr;
 
 	assert(nbr);
 
-	if (nbr->oi) {
-		ifaddr = nbr->oi->address->u.prefix4;
-	}
+	ifaddr = nbr->oi->address->u.prefix4;
 
 	nbraddr = nbr->address.u.prefix4;
 
