@@ -56,7 +56,7 @@ static int pim_mroute_set(struct pim_instance *pim, int enable)
 	/*
 	 * We need to create the VRF table for the pim mroute_socket
 	 */
-	if (pim->vrf_id != VRF_DEFAULT) {
+	if (pim->vrf->vrf_id != VRF_DEFAULT) {
 		frr_with_privs(&pimd_privs) {
 
 			data = pim->vrf->data.l.table_id;
@@ -154,7 +154,7 @@ static int pim_mroute_msg_nocache(int fd, struct interface *ifp,
 	struct pim_interface *pim_ifp = ifp->info;
 	struct pim_upstream *up;
 	struct pim_rpf *rpg;
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 
 	rpg = pim_ifp ? RP(pim_ifp->pim, msg->im_dst) : NULL;
 	/*
@@ -183,7 +183,7 @@ static int pim_mroute_msg_nocache(int fd, struct interface *ifp,
 		return 0;
 	}
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.src = msg->im_src;
 	sg.grp = msg->im_dst;
 
@@ -242,7 +242,7 @@ static int pim_mroute_msg_wholepkt(int fd, struct interface *ifp,
 				   const char *buf)
 {
 	struct pim_interface *pim_ifp;
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 	struct pim_rpf *rpg;
 	const struct ip *ip_hdr;
 	struct pim_upstream *up;
@@ -251,13 +251,13 @@ static int pim_mroute_msg_wholepkt(int fd, struct interface *ifp,
 
 	ip_hdr = (const struct ip *)buf;
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.src = ip_hdr->ip_src;
 	sg.grp = ip_hdr->ip_dst;
 
 	up = pim_upstream_find(pim_ifp->pim, &sg);
 	if (!up) {
-		struct prefix_sg star = sg;
+		pim_sgaddr star = sg;
 		star.src.s_addr = INADDR_ANY;
 
 		up = pim_upstream_find(pim_ifp->pim, &star);
@@ -342,9 +342,9 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 {
 	struct pim_ifchannel *ch;
 	struct pim_interface *pim_ifp;
-	struct prefix_sg sg;
+	pim_sgaddr sg;
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.src = msg->im_src;
 	sg.grp = msg->im_dst;
 
@@ -378,7 +378,7 @@ static int pim_mroute_msg_wrongvif(int fd, struct interface *ifp,
 
 	ch = pim_ifchannel_find(ifp, &sg);
 	if (!ch) {
-		struct prefix_sg star_g = sg;
+		pim_sgaddr star_g = sg;
 		if (PIM_DEBUG_MROUTE)
 			zlog_debug(
 				"%s: WRONGVIF (S,G)=%s could not find channel on interface %s",
@@ -448,12 +448,12 @@ static int pim_mroute_msg_wrvifwhole(int fd, struct interface *ifp,
 	struct pim_instance *pim;
 	struct pim_ifchannel *ch;
 	struct pim_upstream *up;
-	struct prefix_sg star_g;
-	struct prefix_sg sg;
+	pim_sgaddr star_g;
+	pim_sgaddr sg;
 
 	pim_ifp = ifp->info;
 
-	memset(&sg, 0, sizeof(struct prefix_sg));
+	memset(&sg, 0, sizeof(sg));
 	sg.src = ip_hdr->ip_src;
 	sg.grp = ip_hdr->ip_dst;
 
@@ -592,12 +592,9 @@ static int pim_mroute_msg(struct pim_instance *pim, const char *buf,
 	struct pim_interface *pim_ifp;
 	const struct ip *ip_hdr;
 	const struct igmpmsg *msg;
-	char ip_src_str[INET_ADDRSTRLEN] = "";
-	char ip_dst_str[INET_ADDRSTRLEN] = "";
-	char src_str[INET_ADDRSTRLEN] = "<src?>";
-	char grp_str[INET_ADDRSTRLEN] = "<grp?>";
 	struct in_addr ifaddr;
-	struct igmp_sock *igmp;
+	struct gm_sock *igmp;
+	const struct prefix *connected_src;
 
 	if (buf_size < (int)sizeof(struct ip))
 		return 0;
@@ -612,39 +609,43 @@ static int pim_mroute_msg(struct pim_instance *pim, const char *buf,
 		 * the source
 		 * of the IP packet.
 		 */
-		ifp = if_lookup_by_index(ifindex, pim->vrf_id);
+		ifp = if_lookup_by_index(ifindex, pim->vrf->vrf_id);
 
 		if (!ifp || !ifp->info)
 			return 0;
 
+		connected_src = pim_if_connected_to_source(ifp, ip_hdr->ip_src);
+
+		if (!connected_src) {
+			if (PIM_DEBUG_IGMP_PACKETS) {
+				zlog_debug("Recv IGMP packet on interface: %s from a non-connected source: %pI4",
+					   ifp->name, &ip_hdr->ip_src);
+			}
+			return 0;
+		}
+
 		pim_ifp = ifp->info;
-		ifaddr = pim_find_primary_addr(ifp);
-		igmp = pim_igmp_sock_lookup_ifaddr(pim_ifp->igmp_socket_list,
+		ifaddr = connected_src->u.prefix4;
+		igmp = pim_igmp_sock_lookup_ifaddr(pim_ifp->gm_socket_list,
 						   ifaddr);
 
-		if (PIM_DEBUG_MROUTE) {
-			pim_inet4_dump("<src?>", ip_hdr->ip_src, ip_src_str,
-				       sizeof(ip_src_str));
-			pim_inet4_dump("<dst?>", ip_hdr->ip_dst, ip_dst_str,
-				       sizeof(ip_dst_str));
-
+		if (PIM_DEBUG_IGMP_PACKETS) {
 			zlog_debug(
-				"%s(%s): igmp kernel upcall on %s(%p) for %s -> %s",
+				"%s(%s): igmp kernel upcall on %s(%p) for %pI4 -> %pI4",
 				__func__, pim->vrf->name, ifp->name, igmp,
-				ip_src_str, ip_dst_str);
+				&ip_hdr->ip_src, &ip_hdr->ip_dst);
 		}
 		if (igmp)
 			pim_igmp_packet(igmp, (char *)buf, buf_size);
-
+		else if (PIM_DEBUG_IGMP_PACKETS) {
+			zlog_debug("No IGMP socket on interface: %s with connected source: %pFX",
+				   ifp->name, connected_src);
+		}
 	} else if (ip_hdr->ip_p) {
 		if (PIM_DEBUG_MROUTE_DETAIL) {
-			pim_inet4_dump("<src?>", ip_hdr->ip_src, src_str,
-				       sizeof(src_str));
-			pim_inet4_dump("<grp?>", ip_hdr->ip_dst, grp_str,
-				       sizeof(grp_str));
 			zlog_debug(
-				"%s: no kernel upcall proto=%d src: %s dst: %s msg_size=%d",
-				__func__, ip_hdr->ip_p, src_str, grp_str,
+				"%s: no kernel upcall proto=%d src: %pI4 dst: %pI4 msg_size=%d",
+				__func__, ip_hdr->ip_p, &ip_hdr->ip_src, &ip_hdr->ip_dst,
 				buf_size);
 		}
 
@@ -656,15 +657,11 @@ static int pim_mroute_msg(struct pim_instance *pim, const char *buf,
 		if (!ifp)
 			return 0;
 		if (PIM_DEBUG_MROUTE) {
-			pim_inet4_dump("<src?>", msg->im_src, src_str,
-				       sizeof(src_str));
-			pim_inet4_dump("<grp?>", msg->im_dst, grp_str,
-				       sizeof(grp_str));
 			zlog_debug(
-				"%s: pim kernel upcall %s type=%d ip_p=%d from fd=%d for (S,G)=(%s,%s) on %s vifi=%d  size=%d",
+				"%s: pim kernel upcall %s type=%d ip_p=%d from fd=%d for (S,G)=(%pI4,%pI4) on %s vifi=%d  size=%d",
 				__func__, igmpmsgtype2str[msg->im_msgtype],
 				msg->im_msgtype, ip_hdr->ip_p,
-				pim->mroute_socket, src_str, grp_str, ifp->name,
+				pim->mroute_socket, &msg->im_src, &msg->im_dst, ifp->name,
 				msg->im_vif, buf_size);
 		}
 
@@ -672,19 +669,15 @@ static int pim_mroute_msg(struct pim_instance *pim, const char *buf,
 		case IGMPMSG_WRONGVIF:
 			return pim_mroute_msg_wrongvif(pim->mroute_socket, ifp,
 						       msg);
-			break;
 		case IGMPMSG_NOCACHE:
 			return pim_mroute_msg_nocache(pim->mroute_socket, ifp,
 						      msg);
-			break;
 		case IGMPMSG_WHOLEPKT:
 			return pim_mroute_msg_wholepkt(pim->mroute_socket, ifp,
 						       (const char *)msg);
-			break;
 		case IGMPMSG_WRVIFWHOLE:
 			return pim_mroute_msg_wrvifwhole(
 				pim->mroute_socket, ifp, (const char *)msg);
-			break;
 		default:
 			break;
 		}
@@ -1221,14 +1214,13 @@ void pim_mroute_update_counters(struct channel_oil *c_oil)
 	if (!c_oil->installed) {
 		c_oil->cc.lastused = 100 * pim->keep_alive_time;
 		if (PIM_DEBUG_MROUTE) {
-			struct prefix_sg sg;
+			pim_sgaddr sg;
 
 			sg.src = c_oil->oil.mfcc_origin;
 			sg.grp = c_oil->oil.mfcc_mcastgrp;
-			if (PIM_DEBUG_MROUTE)
-				zlog_debug(
-					"Channel%s is not installed no need to collect data from kernel",
-					pim_str_sg_dump(&sg));
+			zlog_debug(
+				"Channel%s is not installed no need to collect data from kernel",
+				pim_str_sg_dump(&sg));
 		}
 		return;
 	}
@@ -1239,7 +1231,7 @@ void pim_mroute_update_counters(struct channel_oil *c_oil)
 
 	pim_zlookup_sg_statistics(c_oil);
 	if (ioctl(pim->mroute_socket, SIOCGETSGCNT, &sgreq)) {
-		struct prefix_sg sg;
+		pim_sgaddr sg;
 
 		sg.src = c_oil->oil.mfcc_origin;
 		sg.grp = c_oil->oil.mfcc_mcastgrp;
